@@ -3,3 +3,6 @@ rlang::`%||%`
 
 #' @importFrom purrr %>%
 purrr::`%>%`
+
+#' @importFrom zeallot %<-%
+zeallot::`%<-%`
