@@ -1,11 +1,9 @@
-from mlflow.anthropic.autolog import patched_class_call
-from mlflow.utils.annotations import experimental
+from mlflow.anthropic.autolog import async_patched_class_call, patched_class_call
 from mlflow.utils.autologging_utils import autologging_integration, safe_patch
 
 FLAVOR_NAME = "anthropic"
 
 
-@experimental
 @autologging_integration(FLAVOR_NAME)
 def autolog(
     log_traces: bool = True,
@@ -14,7 +12,7 @@ def autolog(
 ):
     """
     Enables (or disables) and configures autologging from Anthropic to MLflow.
-    Only synchronous calls are supported. Asynchnorous APIs and streaming are not recorded.
+    Only synchronous calls and asynchronous APIs are supported. Streaming is not recorded.
 
     Args:
         log_traces: If ``True``, traces are logged for Anthropic models.
@@ -23,7 +21,7 @@ def autolog(
         silent: If ``True``, suppress all event logs and warnings from MLflow during Anthropic
             autologging. If ``False``, show all events and warnings.
     """
-    from anthropic.resources import Messages
+    from anthropic.resources import AsyncMessages, Messages
 
     safe_patch(
         FLAVOR_NAME,
@@ -31,3 +29,10 @@ def autolog(
         "create",
         patched_class_call,
     )
+
+    safe_patch(
+        FLAVOR_NAME,
+        AsyncMessages,
+        "create",
+        async_patched_class_call,
+    )
