@@ -1,7 +1,7 @@
 import json
 import posixpath
 import warnings
-from typing import Any, Iterator, Optional
+from typing import Any, Iterator
 
 from mlflow.deployments import BaseDeploymentClient
 from mlflow.deployments.constants import (
@@ -13,7 +13,7 @@ from mlflow.environment_variables import (
 )
 from mlflow.exceptions import MlflowException
 from mlflow.utils import AttrDict
-from mlflow.utils.annotations import deprecated, experimental
+from mlflow.utils.annotations import deprecated
 from mlflow.utils.databricks_utils import get_databricks_host_creds
 from mlflow.utils.rest_utils import augmented_raise_for_status, http_request
 
@@ -40,7 +40,6 @@ class DatabricksEndpoint(AttrDict):
     """
 
 
-@experimental
 class DatabricksDeploymentClient(BaseDeploymentClient):
     """
     Client for interacting with Databricks serving endpoints.
@@ -125,9 +124,9 @@ class DatabricksDeploymentClient(BaseDeploymentClient):
         *,
         method: str,
         prefix: str = "/api/2.0",
-        route: Optional[str] = None,
-        json_body: Optional[dict[str, Any]] = None,
-        timeout: Optional[int] = None,
+        route: str | None = None,
+        json_body: dict[str, Any] | None = None,
+        timeout: int | None = None,
     ):
         call_kwargs = {}
         if method.lower() == "get":
@@ -153,9 +152,9 @@ class DatabricksDeploymentClient(BaseDeploymentClient):
         *,
         method: str,
         prefix: str = "/api/2.0",
-        route: Optional[str] = None,
-        json_body: Optional[dict[str, Any]] = None,
-        timeout: Optional[int] = None,
+        route: str | None = None,
+        json_body: dict[str, Any] | None = None,
+        timeout: int | None = None,
     ) -> Iterator[str]:
         call_kwargs = {}
         if method.lower() == "get":
@@ -178,13 +177,15 @@ class DatabricksDeploymentClient(BaseDeploymentClient):
 
         # Streaming response content are composed of multiple lines.
         # Each line format depends on specific endpoint
+        # Explicitly set the encoding to `utf-8` so the `decode_unicode` in the next line
+        # will decode correctly
+        response.encoding = "utf-8"
         return (
             line.strip()
             for line in response.iter_lines(decode_unicode=True)
             if line.strip()  # filter out keep-alive new lines
         )
 
-    @experimental
     def predict(self, deployment_name=None, inputs=None, endpoint=None):
         """
         Query a serving endpoint with the provided model inputs.
@@ -244,7 +245,6 @@ class DatabricksDeploymentClient(BaseDeploymentClient):
             timeout=MLFLOW_DEPLOYMENT_PREDICT_TIMEOUT.get(),
         )
 
-    @experimental
     def predict_stream(
         self, deployment_name=None, inputs=None, endpoint=None
     ) -> Iterator[dict[str, Any]]:
@@ -326,7 +326,6 @@ class DatabricksDeploymentClient(BaseDeploymentClient):
 
             yield json.loads(value)
 
-    @experimental
     def create_endpoint(self, name=None, config=None, route_optimized=False):
         """
         Create a new serving endpoint with the provided name and configuration.
@@ -560,7 +559,6 @@ class DatabricksDeploymentClient(BaseDeploymentClient):
                 method="PUT", route=posixpath.join(endpoint, "config"), json_body=config
             )
 
-    @experimental
     def update_endpoint_config(self, endpoint, config):
         """
         Update the configuration of a specified serving endpoint. See
@@ -620,7 +618,6 @@ class DatabricksDeploymentClient(BaseDeploymentClient):
             method="PUT", route=posixpath.join(endpoint, "config"), json_body=config
         )
 
-    @experimental
     def update_endpoint_tags(self, endpoint, config):
         """
         Update the tags of a specified serving endpoint. See
@@ -650,7 +647,6 @@ class DatabricksDeploymentClient(BaseDeploymentClient):
             method="PATCH", route=posixpath.join(endpoint, "tags"), json_body=config
         )
 
-    @experimental
     def update_endpoint_rate_limits(self, endpoint, config):
         """
         Update the rate limits of a specified serving endpoint.
@@ -686,7 +682,6 @@ class DatabricksDeploymentClient(BaseDeploymentClient):
             method="PUT", route=posixpath.join(endpoint, "rate-limits"), json_body=config
         )
 
-    @experimental
     def update_endpoint_ai_gateway(self, endpoint, config):
         """
         Update the AI Gateway configuration of a specified serving endpoint.
@@ -733,7 +728,6 @@ class DatabricksDeploymentClient(BaseDeploymentClient):
             method="PUT", route=posixpath.join(endpoint, "ai-gateway"), json_body=config
         )
 
-    @experimental
     def delete_endpoint(self, endpoint):
         """
         Delete a specified serving endpoint.
@@ -757,7 +751,6 @@ class DatabricksDeploymentClient(BaseDeploymentClient):
         """
         return self._call_endpoint(method="DELETE", route=endpoint)
 
-    @experimental
     def list_endpoints(self):
         """
         Retrieve all serving endpoints.
@@ -792,7 +785,6 @@ class DatabricksDeploymentClient(BaseDeploymentClient):
         """
         return self._call_endpoint(method="GET").endpoints
 
-    @experimental
     def get_endpoint(self, endpoint):
         """
         Get a specified serving endpoint.
