@@ -4,14 +4,19 @@ import json
 import logging
 import re
 from dataclasses import dataclass
-from typing import Any, Optional, Union
+from typing import TYPE_CHECKING, Any, Literal
 
 from mlflow.entities._mlflow_object import _MlflowObject
 from mlflow.entities.span import Span, SpanType
 from mlflow.entities.trace_data import TraceData
 from mlflow.entities.trace_info import TraceInfo
+from mlflow.entities.trace_info_v2 import TraceInfoV2
 from mlflow.exceptions import MlflowException
 from mlflow.protos.databricks_pb2 import INVALID_PARAMETER_VALUE
+from mlflow.protos.service_pb2 import Trace as ProtoTrace
+
+if TYPE_CHECKING:
+    from mlflow.entities.assessment import Assessment
 
 _logger = logging.getLogger(__name__)
 
@@ -28,8 +33,12 @@ class Trace(_MlflowObject):
     info: TraceInfo
     data: TraceData
 
+    def __post_init__(self):
+        if isinstance(self.info, TraceInfoV2):
+            self.info = self.info.to_v3(request=self.data.request, response=self.data.response)
+
     def __repr__(self) -> str:
-        return f"Trace(request_id={self.info.request_id})"
+        return f"Trace(trace_id={self.info.trace_id})"
 
     def to_dict(self) -> dict[str, Any]:
         return {"info": self.info.to_dict(), "data": self.data.to_dict()}
@@ -101,16 +110,18 @@ class Trace(_MlflowObject):
 
     def to_pandas_dataframe_row(self) -> dict[str, Any]:
         return {
-            "request_id": self.info.request_id,
-            "trace": self,
-            "timestamp_ms": self.info.timestamp_ms,
-            "status": self.info.status,
-            "execution_time_ms": self.info.execution_time_ms,
+            "trace_id": self.info.trace_id,
+            "trace": self.to_json(),  # json string to be compatible with Spark DataFrame
+            "client_request_id": self.info.client_request_id,
+            "state": self.info.state,
+            "request_time": self.info.request_time,
+            "execution_duration": self.info.execution_duration,
             "request": self._deserialize_json_attr(self.data.request),
             "response": self._deserialize_json_attr(self.data.response),
-            "request_metadata": self.info.request_metadata,
-            "spans": [span.to_dict() for span in self.data.spans],
+            "trace_metadata": self.info.trace_metadata,
             "tags": self.info.tags,
+            "spans": [span.to_dict() for span in self.data.spans],
+            "assessments": [assessment.to_dictionary() for assessment in self.info.assessments],
         }
 
     def _deserialize_json_attr(self, value: str):
@@ -121,7 +132,10 @@ class Trace(_MlflowObject):
             return value
 
     def search_spans(
-        self, span_type: Optional[SpanType] = None, name: Optional[Union[str, re.Pattern]] = None
+        self,
+        span_type: SpanType | None = None,
+        name: str | re.Pattern | None = None,
+        span_id: str | None = None,
     ) -> list[Span]:
         """
         Search for spans that match the given criteria within the trace.
@@ -129,6 +143,7 @@ class Trace(_MlflowObject):
         Args:
             span_type: The type of the span to search for.
             name: The name of the span to search for. This can be a string or a regular expression.
+            span_id: The ID of the span to search for.
 
         Returns:
             A list of spans that match the given criteria.
@@ -166,7 +181,8 @@ class Trace(_MlflowObject):
 
             # Run the function and get the trace
             y = run(2)
-            trace = mlflow.get_last_active_trace()
+            trace_id = mlflow.get_last_active_trace_id()
+            trace = mlflow.get_trace(trace_id)
 
             # 1. Search spans by name (exact match)
             spans = trace.search_spans(name="add_one")
@@ -215,19 +231,87 @@ class Trace(_MlflowObject):
                     error_code=INVALID_PARAMETER_VALUE,
                 )
 
-        return [span for span in self.data.spans if _match_name(span) and _match_type(span)]
+        def _match_id(span: Span) -> bool:
+            if span_id is None:
+                return True
+            else:
+                return span.span_id == span_id
+
+        return [
+            span
+            for span in self.data.spans
+            if _match_name(span) and _match_type(span) and _match_id(span)
+        ]
+
+    def search_assessments(
+        self,
+        name: str | None = None,
+        *,
+        span_id: str | None = None,
+        all: bool = False,
+        type: Literal["expectation", "feedback"] | None = None,
+    ) -> list["Assessment"]:
+        """
+        Get assessments for a given name / span ID. By default, this only returns assessments
+        that are valid (i.e. have not been overridden by another assessment). To return all
+        assessments, specify `all=True`.
+
+        Args:
+            name: The name of the assessment to get. If not provided, this will match
+                all assessment names.
+            span_id: The span ID to get assessments for.
+                If not provided, this will match all spans.
+            all: If True, return all assessments regardless of validity.
+            type: The type of assessment to get (one of "feedback" or "expectation").
+                If not provided, this will match all assessment types.
+
+        Returns:
+            A list of assessments that meet the given conditions.
+        """
+
+        def validate_type(assessment: Assessment) -> bool:
+            from mlflow.entities.assessment import Expectation, Feedback
+
+            if type == "expectation":
+                return isinstance(assessment, Expectation)
+            elif type == "feedback":
+                return isinstance(assessment, Feedback)
+
+            return True
+
+        return [
+            assessment
+            for assessment in self.info.assessments
+            if (name is None or assessment.name == name)
+            and (span_id is None or assessment.span_id == span_id)
+            # valid defaults to true, so Nones are valid
+            and (all or assessment.valid in (True, None))
+            and (type is None or validate_type(assessment))
+        ]
 
     @staticmethod
     def pandas_dataframe_columns() -> list[str]:
         return [
-            "request_id",
+            "trace_id",
             "trace",
-            "timestamp_ms",
-            "status",
-            "execution_time_ms",
+            "client_request_id",
+            "state",
+            "request_time",
+            "execution_duration",
             "request",
             "response",
-            "request_metadata",
-            "spans",
+            "trace_metadata",
             "tags",
+            "spans",
+            "assessments",
         ]
+
+    def to_proto(self):
+        """
+        Convert into a proto object to sent to the MLflow backend.
+
+        NB: The Trace definition in MLflow backend doesn't include the `data` field,
+            but rather only contains TraceInfoV3.
+        """
+
+        return ProtoTrace(trace_info=self.info.to_proto())
