@@ -24,7 +24,7 @@ import logging
 import os
 import tempfile
 from copy import deepcopy
-from typing import Any, Optional
+from typing import Any
 
 import yaml
 from packaging.version import Version
@@ -44,6 +44,7 @@ from mlflow.sklearn import _SklearnTrainingSession
 from mlflow.tracking._model_registry import DEFAULT_AWAIT_MAX_SLEEP_SECONDS
 from mlflow.tracking.artifact_utils import _download_artifact_from_uri
 from mlflow.tracking.context import registry as context_registry
+from mlflow.tracking.fluent import _initialize_logged_model
 from mlflow.utils import _get_fully_qualified_class_name
 from mlflow.utils.arguments_utils import _get_arg_names
 from mlflow.utils.autologging_utils import (
@@ -272,7 +273,7 @@ def _save_model(lgb_model, model_path):
 @format_docstring(LOG_MODEL_PARAM_DOCS.format(package_name=FLAVOR_NAME))
 def log_model(
     lgb_model,
-    artifact_path,
+    artifact_path: str | None = None,
     conda_env=None,
     code_paths=None,
     registered_model_name=None,
@@ -282,6 +283,12 @@ def log_model(
     pip_requirements=None,
     extra_pip_requirements=None,
     metadata=None,
+    name: str | None = None,
+    params: dict[str, Any] | None = None,
+    tags: dict[str, Any] | None = None,
+    model_type: str | None = None,
+    step: int = 0,
+    model_id: str | None = None,
     **kwargs,
 ):
     """
@@ -290,7 +297,7 @@ def log_model(
     Args:
         lgb_model: LightGBM model (an instance of `lightgbm.Booster`_) or
             models that implement the `scikit-learn API`_  to be saved.
-        artifact_path: Run-relative artifact path.
+        artifact_path: Deprecated. Use `name` instead.
         conda_env: {{ conda_env }}
         code_paths: {{ code_paths }}
         registered_model_name: If given, create a model version under
@@ -305,6 +312,12 @@ def log_model(
         pip_requirements: {{ pip_requirements }}
         extra_pip_requirements: {{ extra_pip_requirements }}
         metadata: {{ metadata }}
+        name: {{ name }}
+        params: {{ params }}
+        tags: {{ tags }}
+        model_type: {{ model_type }}
+        step: {{ step }}
+        model_id: {{ model_id }}
         kwargs: kwargs to pass to `lightgbm.Booster.save_model`_ method.
 
     Returns:
@@ -335,7 +348,9 @@ def log_model(
         # Log the model
         artifact_path = "model"
         with mlflow.start_run():
-            model_info = mlflow.lightgbm.log_model(model, artifact_path, signature=signature)
+            model_info = mlflow.lightgbm.log_model(
+                model, name=artifact_path, signature=signature
+            )
 
         # Fetch the logged model artifacts
         print(f"run_id: {run.info.run_id}")
@@ -354,6 +369,7 @@ def log_model(
     """
     return Model.log(
         artifact_path=artifact_path,
+        name=name,
         flavor=mlflow.lightgbm,
         registered_model_name=registered_model_name,
         lgb_model=lgb_model,
@@ -365,6 +381,11 @@ def log_model(
         pip_requirements=pip_requirements,
         extra_pip_requirements=extra_pip_requirements,
         metadata=metadata,
+        params=params,
+        tags=tags,
+        model_type=model_type,
+        step=step,
+        model_id=model_id,
         **kwargs,
     )
 
@@ -478,7 +499,7 @@ class _LGBModelWrapper:
         """
         return self.lgb_model
 
-    def predict(self, dataframe, params: Optional[dict[str, Any]] = None):
+    def predict(self, dataframe, params: dict[str, Any] | None = None):
         """
         Args:
             dataframe: Model input data.
@@ -781,7 +802,10 @@ def autolog(
                     "Failed to log dataset information to MLflow Tracking. Reason: %s", e
                 )
 
-        with batch_metrics_logger(run_id) as metrics_logger:
+        model_id = None
+        if _log_models:
+            model_id = _initialize_logged_model("model", flavor=FLAVOR_NAME).model_id
+        with batch_metrics_logger(run_id, model_id=model_id) as metrics_logger:
             callback = record_eval_results(eval_results, metrics_logger)
             if num_pos_args >= callbacks_index + 1:
                 tmp_list = list(args)
@@ -795,27 +819,29 @@ def autolog(
             # training model
             model = original(*args, **kwargs)
 
-            # If early stopping is activated, logging metrics at the best iteration
-            # as extra metrics with the max step + 1.
-            early_stopping = model.best_iteration > 0
-            if early_stopping:
-                extra_step = len(eval_results)
-                autologging_client.log_metrics(
-                    run_id=mlflow.active_run().info.run_id,
-                    metrics={
-                        "stopped_iteration": extra_step,
-                        # best_iteration is set even if training does not stop early.
-                        "best_iteration": model.best_iteration,
-                    },
-                )
-                # iteration starts from 1 in LightGBM.
-                last_iter_results = eval_results[model.best_iteration - 1]
-                autologging_client.log_metrics(
-                    run_id=mlflow.active_run().info.run_id,
-                    metrics=last_iter_results,
-                    step=extra_step,
-                )
-                early_stopping_logging_operations = autologging_client.flush(synchronous=False)
+        # If early stopping is activated, logging metrics at the best iteration
+        # as extra metrics with the max step + 1.
+        early_stopping = model.best_iteration > 0
+        if early_stopping:
+            extra_step = len(eval_results)
+            autologging_client.log_metrics(
+                run_id=mlflow.active_run().info.run_id,
+                metrics={
+                    "stopped_iteration": extra_step,
+                    # best_iteration is set even if training does not stop early.
+                    "best_iteration": model.best_iteration,
+                },
+                model_id=model_id,
+            )
+            # iteration starts from 1 in LightGBM.
+            last_iter_results = eval_results[model.best_iteration - 1]
+            autologging_client.log_metrics(
+                run_id=mlflow.active_run().info.run_id,
+                metrics=last_iter_results,
+                step=extra_step,
+                model_id=model_id,
+            )
+            early_stopping_logging_operations = autologging_client.flush(synchronous=False)
 
         # logging feature importance as artifacts.
         for imp_type in ["split", "gain"]:
@@ -869,6 +895,7 @@ def autolog(
                 signature=signature,
                 input_example=input_example,
                 registered_model_name=registered_model_name,
+                model_id=model_id,
             )
 
         param_logging_operations.await_completion()
