@@ -5,7 +5,7 @@ from mlflow.utils.autologging_utils import autologging_integration, safe_patch
 FLAVOR_NAME = "mistral"
 
 
-@experimental
+@experimental(version="2.21.0")
 @autologging_integration(FLAVOR_NAME)
 def autolog(
     log_traces: bool = True,
