@@ -33,7 +33,8 @@ def _cumulative_gain_curve(y_true, y_score, pos_label=None):
         ValueError: If `y_true` is not composed of 2 classes. The Cumulative
             Gain Chart is only relevant in binary classification.
     """
-    y_true, y_score = np.asarray(y_true), np.asarray(y_score)
+    y_true = np.asarray(y_true)
+    y_score = np.asarray(y_score)
 
     # ensure binary classification if pos_label is not specified
     classes = np.unique(y_true)
@@ -135,9 +136,7 @@ def plot_lift_curve(
 
     classes = np.unique(y_true)
     if len(classes) != 2:
-        raise ValueError(
-            "Cannot calculate Lift Curve for data with {} category/ies".format(len(classes))
-        )
+        raise ValueError(f"Cannot calculate Lift Curve for data with {len(classes)} category/ies")
 
     # Compute Cumulative Gain Curves
     percentages, gains1 = _cumulative_gain_curve(y_true, y_probas[:, 0], classes[0])
@@ -155,14 +154,14 @@ def plot_lift_curve(
 
     ax.set_title(title, fontsize=title_fontsize)
 
-    label0 = "Class {}".format(classes[0])
-    label1 = "Class {}".format(classes[1])
+    label0 = f"Class {classes[0]}"
+    label1 = f"Class {classes[1]}"
     # show (positive) next to the positive class in the legend
     if pos_label:
         if pos_label == classes[0]:
-            label0 = "Class {} (positive)".format(classes[0])
+            label0 = f"Class {classes[0]} (positive)"
         elif pos_label == classes[1]:
-            label1 = "Class {} (positive)".format(classes[1])
+            label1 = f"Class {classes[1]} (positive)"
         # do not mark positive class if pos_label is not in classes
 
     ax.plot(percentages, gains1, lw=3, label=label0)
