@@ -2,10 +2,12 @@ import logging
 import os
 import re
 import subprocess
+import sys
 
+from mlflow.exceptions import MlflowException
 from mlflow.models import FlavorBackend
-from mlflow.utils.string_utils import quote
 from mlflow.tracking.artifact_utils import _download_artifact_from_uri
+from mlflow.utils.string_utils import quote
 
 _logger = logging.getLogger(__name__)
 
@@ -16,21 +18,47 @@ class RFuncBackend(FlavorBackend):
     Predict and serve locally models with 'crate' flavor.
     """
 
-    def build_image(self, model_uri, image_name, install_mlflow, mlflow_home, enable_mlserver):
+    def build_image(
+        self,
+        model_uri,
+        image_name,
+        install_java=False,
+        install_mlflow=False,
+        mlflow_home=None,
+        enable_mlserver=False,
+        base_image=None,
+    ):
         pass
 
     def generate_dockerfile(
-        self, model_uri, output_path, install_mlflow, mlflow_home, enable_mlserver
+        self,
+        model_uri,
+        output_dir,
+        install_java=False,
+        install_mlflow=False,
+        mlflow_home=None,
+        enable_mlserver=False,
+        base_image=None,
     ):
         pass
 
     version_pattern = re.compile(r"version ([0-9]+\.[0-9]+\.[0-9]+)")
 
-    def predict(self, model_uri, input_path, output_path, content_type):
+    def predict(
+        self,
+        model_uri,
+        input_path,
+        output_path,
+        content_type,
+        pip_requirements_override=None,
+        extra_envs=None,
+    ):
         """
         Generate predictions using R model saved with MLflow.
         Return the prediction results as a JSON.
         """
+        if pip_requirements_override is not None:
+            raise MlflowException("pip_requirements_override is not supported in the R backend.")
         model_path = _download_artifact_from_uri(model_uri)
         str_cmd = (
             "mlflow:::mlflow_rfunc_predict(model_path = '{0}', input_path = {1}, "
@@ -42,7 +70,7 @@ class RFuncBackend(FlavorBackend):
             _str_optional(output_path),
             _str_optional(content_type),
         )
-        _execute(command)
+        _execute(command, extra_envs=extra_envs)
 
     def serve(
         self,
@@ -99,9 +127,10 @@ class RFuncBackend(FlavorBackend):
         return version[0] > 3 or version[0] == 3 and version[1] >= 3
 
 
-def _execute(command):
+def _execute(command, extra_envs=None):
     env = os.environ.copy()
-    import sys
+    if extra_envs:
+        env.update(extra_envs)
 
     process = subprocess.Popen(
         ["Rscript", "-e", command],
@@ -116,4 +145,4 @@ def _execute(command):
 
 
 def _str_optional(s):
-    return "NULL" if s is None else "'{}'".format(quote(str(s)))
+    return "NULL" if s is None else f"'{quote(str(s))}'"
