@@ -1,13 +1,11 @@
 """add model version tags table
 
-Revision ID: 27a6a02d2cf1
-Revises: 728d730b5ebd
 Create Date: 2020-06-26 13:30:27.611086
 
 """
-from alembic import op
-import sqlalchemy as sa
 
+import sqlalchemy as sa
+from alembic import op
 
 # revision identifiers, used by Alembic.
 from mlflow.store.model_registry.dbmodels.models import SqlModelVersionTag
