@@ -1,4 +1,5 @@
 from abc import ABCMeta, abstractmethod
+
 from mlflow.utils.annotations import developer_stable
 
 
@@ -8,8 +9,8 @@ class RequestHeaderProvider:
     Abstract base class for specifying custom request headers to add to outgoing requests
     (e.g. request headers specifying the environment from which mlflow is running).
 
-    When a request is sent, Mlflow will iterate through all registered RequestHeaderProviders.
-    For each provider where ``in_context`` returns ``True``, Mlflow calls the ``request_headers``
+    When a request is sent, MLflow will iterate through all registered RequestHeaderProviders.
+    For each provider where ``in_context`` returns ``True``, MLflow calls the ``request_headers``
     method on the provider to compute request headers.
 
     All resulting request headers will then be merged together and sent with the request.
@@ -19,18 +20,17 @@ class RequestHeaderProvider:
 
     @abstractmethod
     def in_context(self):
-        """
-        Determine if MLflow is running in this context.
+        """Determine if MLflow is running in this context.
+
+        Returns:
+            bool indicating if in this context.
 
-        :return: bool indicating if in this context
         """
-        pass
 
     @abstractmethod
     def request_headers(self):
-        """
-        Generate context-specific request headers.
+        """Generate context-specific request headers.
 
-        :return: dict of request headers
+        Returns:
+            dict of request headers.
         """
-        pass
