@@ -2,23 +2,21 @@ import abc
 import functools
 import inspect
 import itertools
-import typing
 import uuid
-from abc import abstractmethod
-from contextlib import contextmanager
-from typing import Optional
+from contextlib import asynccontextmanager, contextmanager
+from typing import Any, Callable, NamedTuple
 
 import mlflow
 import mlflow.utils.autologging_utils
 from mlflow.entities.run_status import RunStatus
 from mlflow.environment_variables import _MLFLOW_AUTOLOGGING_TESTING
-from mlflow.tracking.client import MlflowClient
+from mlflow.exceptions import MlflowException
 from mlflow.utils import gorilla, is_iterator
 from mlflow.utils.autologging_utils import _logger
-from mlflow.utils.autologging_utils.events import AutologgingEventLogger
+from mlflow.utils.autologging_utils.events import AutologgingEventLoggerWrapper
 from mlflow.utils.autologging_utils.logging_and_warnings import (
-    set_mlflow_events_and_warnings_behavior_globally,
-    set_non_mlflow_warnings_behavior_for_current_thread,
+    MlflowEventsAndWarningsBehaviorGlobally,
+    NonMlflowWarningsBehaviorForCurrentThread,
 )
 from mlflow.utils.mlflow_tags import MLFLOW_AUTOLOGGING
 
@@ -31,6 +29,9 @@ _AUTOLOGGING_PATCHES = {}
 _ATTRIBUTE_EXCEPTION_SAFE = "exception_safe"
 
 
+_ERROR_MSG = "Encountered unexpected error during {} autologging: {}"
+
+
 def exception_safe_function_for_class(function):
     """
     Wraps the specified function with broad exception handling to guard
@@ -129,54 +130,6 @@ ExceptionSafeClass = _exception_safe_class_factory(type)
 ExceptionSafeAbstractClass = _exception_safe_class_factory(abc.ABCMeta)
 
 
-class PatchFunction:
-    """
-    Base class representing a function patch implementation with a callback for error handling.
-    `PatchFunction` should be subclassed and used in conjunction with `safe_patch` to
-    safely modify the implementation of a function. Subclasses of `PatchFunction` should
-    use `_patch_implementation` to define modified ("patched") function implementations and
-    `_on_exception` to define cleanup logic when `_patch_implementation` terminates due
-    to an unhandled exception.
-    """
-
-    @abstractmethod
-    def _patch_implementation(self, original, *args, **kwargs):
-        """
-        Invokes the patch function code.
-
-        Args:
-            original: The original, underlying function over which the `PatchFunction`
-                is being applied.
-            *args: The positional arguments passed to the original function.
-            **kwargs: The keyword arguments passed to the original function.
-        """
-
-    @abstractmethod
-    def _on_exception(self, exception):
-        """Called when an unhandled standard Python exception (i.e. an exception inheriting from
-        `Exception`) or a `KeyboardInterrupt` prematurely terminates the execution of
-        `_patch_implementation`.
-
-        Args:
-            exception: The unhandled exception thrown by `_patch_implementation`.
-        """
-
-    @classmethod
-    def call(cls, original, *args, **kwargs):
-        return cls().__call__(original, *args, **kwargs)
-
-    def __call__(self, original, *args, **kwargs):
-        try:
-            return self._patch_implementation(original, *args, **kwargs)
-        except (Exception, KeyboardInterrupt) as e:
-            try:
-                self._on_exception(e)
-            finally:
-                # Regardless of what happens during the `_on_exception` callback, reraise
-                # the original implementation exception once the callback completes
-                raise e
-
-
 def with_managed_run(autologging_integration, patch_function, tags=None):
     """Given a `patch_function`, returns an `augmented_patch_function` that wraps the execution of
     `patch_function` with an active MLflow run. The following properties apply:
@@ -197,11 +150,11 @@ def with_managed_run(autologging_integration, patch_function, tags=None):
     Args:
         autologging_integration: The autologging integration associated
             with the `patch_function`.
-        patch_function: A `PatchFunction` class definition or a function object
-            compatible with `safe_patch`.
+        patch_function: A function object compatible with `safe_patch`.
         tags: A dictionary of string tags to set on each managed run created during the
             execution of `patch_function`.
     """
+    from mlflow.tracking.fluent import active_run
     from mlflow.utils.autologging_utils import _has_active_training_session
 
     def create_managed_run():
@@ -215,61 +168,30 @@ def with_managed_run(autologging_integration, patch_function, tags=None):
         )
         return managed_run
 
-    if inspect.isclass(patch_function):
+    def patch_with_managed_run(original, *args, **kwargs):
+        managed_run = None
+        # If there is an active training session but there is no active run
+        # in current thread, it means the thread is spawned by `estimator.fit`
+        # as a worker thread, we should disable autologging in
+        # these worker threads, so skip creating managed run.
+        if not active_run() and not _has_active_training_session():
+            managed_run = create_managed_run()
 
-        class PatchWithManagedRun(patch_function):
-            def __init__(self):
-                super().__init__()
-                self.managed_run = None
+        try:
+            result = patch_function(original, *args, **kwargs)
+        except (Exception, KeyboardInterrupt):
+            # In addition to standard Python exceptions, handle keyboard interrupts to ensure
+            # that runs are terminated if a user prematurely interrupts training execution
+            # (e.g. via sigint / ctrl-c)
+            if managed_run:
+                mlflow.end_run(RunStatus.to_string(RunStatus.FAILED))
+            raise
+        else:
+            if managed_run:
+                mlflow.end_run(RunStatus.to_string(RunStatus.FINISHED))
+            return result
 
-            def _patch_implementation(self, original, *args, **kwargs):
-                # If there is an active training session but there is no active run
-                # in current thread, it means the thread is spawned by `estimator.fit`
-                # as a worker thread, we should disable autologging in
-                # these worker threads, so skip creating managed run.
-                if not mlflow.active_run() and not _has_active_training_session():
-                    self.managed_run = create_managed_run()
-
-                result = super()._patch_implementation(original, *args, **kwargs)
-
-                if self.managed_run:
-                    mlflow.end_run(RunStatus.to_string(RunStatus.FINISHED))
-
-                return result
-
-            def _on_exception(self, e):
-                if self.managed_run:
-                    mlflow.end_run(RunStatus.to_string(RunStatus.FAILED))
-                super()._on_exception(e)
-
-        return PatchWithManagedRun
-
-    else:
-
-        def patch_with_managed_run(original, *args, **kwargs):
-            managed_run = None
-            # If there is an active training session but there is no active run
-            # in current thread, it means the thread is spawned by `estimator.fit`
-            # as a worker thread, we should disable autologging in
-            # these worker threads, so skip creating managed run.
-            if not mlflow.active_run() and not _has_active_training_session():
-                managed_run = create_managed_run()
-
-            try:
-                result = patch_function(original, *args, **kwargs)
-            except (Exception, KeyboardInterrupt):
-                # In addition to standard Python exceptions, handle keyboard interrupts to ensure
-                # that runs are terminated if a user prematurely interrupts training execution
-                # (e.g. via sigint / ctrl-c)
-                if managed_run:
-                    mlflow.end_run(RunStatus.to_string(RunStatus.FAILED))
-                raise
-            else:
-                if managed_run:
-                    mlflow.end_run(RunStatus.to_string(RunStatus.FINISHED))
-                return result
-
-        return patch_with_managed_run
+    return patch_with_managed_run
 
 
 def is_testing():
@@ -326,11 +248,9 @@ def safe_patch(
             patch.
         destination: The Python class on which the patch is being defined.
         function_name: The name of the function to patch on the specified `destination` class.
-        patch_function: The patched function code to apply. This is either a `PatchFunction`
-            class definition or a function object. If it is a function object, the
-            first argument should be reserved for an `original` method argument
-            representing the underlying / original function. Subsequent arguments
-            should be identical to those of the original function being patched.
+        patch_function: The patched function code to apply. The first argument should be reserved
+            for an `original` argument representing the underlying / original function. Subsequent
+            arguments should be identical to those of the original function being patched.
         manage_run: If `True`, applies the `with_managed_run` wrapper to the specified
             `patch_function`, which automatically creates & terminates an MLflow
             active run during patch code execution if necessary. If `False`,
@@ -338,9 +258,18 @@ def safe_patch(
             `patch_function`.
         extra_tags: A dictionary of extra tags to set on each managed run created by autologging.
     """
+    from mlflow.tracking.fluent import active_run
     from mlflow.utils.autologging_utils import autologging_is_disabled, get_autologging_config
 
+    # NB: Checking the signature of the patch function rather than original, so that we don't
+    # accidentally change the behavior of existing patches that may use sync patch function
+    # for async original functions (e.g. LangChain).
+    is_async_function = inspect.iscoroutinefunction(patch_function)
+
     if manage_run:
+        if is_async_function:
+            raise MlflowException("manage_run parameter is not supported for async functions.")
+
         tags = _resolve_extra_tags(autologging_integration, extra_tags)
         patch_function = with_managed_run(
             autologging_integration,
@@ -348,12 +277,6 @@ def safe_patch(
             tags=tags,
         )
 
-    patch_is_class = inspect.isclass(patch_function)
-    if patch_is_class:
-        assert issubclass(patch_function, PatchFunction)
-    else:
-        assert callable(patch_function)
-
     original_fn = gorilla.get_original_attribute(
         destination, function_name, bypass_descriptor_protocol=False
     )
@@ -364,6 +287,9 @@ def safe_patch(
     if original_fn != raw_original_obj:
         raise RuntimeError(f"Unsupported patch on {destination}.{function_name}")
     elif isinstance(original_fn, property):
+        if is_async_function:
+            raise MlflowException("Patching async property methods is not supported.")
+
         is_property_method = True
 
         # For property decorated methods (a kind of method delegation), e.g.
@@ -401,6 +327,10 @@ def safe_patch(
         Exceptions thrown from the underlying / original function are propagated to the caller,
         while exceptions thrown from other parts of `patch_function` are caught and logged as
         warnings.
+
+        NB: PLEASE BE SUPER CAREFUL WHEN MODIFYING THIS FUNCTION. IT IS USED IN A WIDE VARIETY
+        OF CONTEXTX AND CRITICAL PATH IN DBR/MLR BY DEFAULT. ANY BUG HERE CAN BREAK USERS'
+        WORKLOAD WITHOUT THEM TAKING ANY ACTION.
         """
         # Reroute warnings encountered during the patch function implementation to an MLflow event
         # logger, and enforce silent mode if applicable (i.e. if the corresponding autologging
@@ -415,7 +345,7 @@ def safe_patch(
         # during model serialization by ML frameworks such as scikit-learn
         is_silent_mode = get_autologging_config(autologging_integration, "silent", False)
         with (
-            set_mlflow_events_and_warnings_behavior_globally(
+            MlflowEventsAndWarningsBehaviorGlobally(
                 # MLflow warnings emitted during autologging training sessions are likely not
                 # actionable and result from the autologging implementation invoking another MLflow
                 # API. Accordingly, we reroute these warnings to the MLflow event logger with level
@@ -425,7 +355,7 @@ def safe_patch(
                 disable_event_logs=is_silent_mode,
                 disable_warnings=is_silent_mode,
             ),
-            set_non_mlflow_warnings_behavior_for_current_thread(
+            NonMlflowWarningsBehaviorForCurrentThread(
                 # non-MLflow Warnings emitted during the autologging preamble (before the original /
                 # underlying ML function is called) and postamble (after the original / underlying
                 # ML function is called) are likely not actionable and result from the autologging
@@ -438,13 +368,13 @@ def safe_patch(
             ),
         ):
             if is_testing():
-                preexisting_run_for_testing = mlflow.active_run()
+                preexisting_run_for_testing = active_run()
 
             # Whether or not to exclude autologged content from user-created fluent runs
             # (i.e. runs created manually via `mlflow.start_run()`)
             exclusive = get_autologging_config(autologging_integration, "exclusive", False)
             user_created_fluent_run_is_active = (
-                mlflow.active_run() and not _AutologgingSessionManager.active_session()
+                active_run() and not _AutologgingSessionManager.active_session()
             )
             active_session_failed = (
                 _AutologgingSessionManager.active_session() is not None
@@ -465,7 +395,7 @@ def safe_patch(
                 # fluent run is active, call the original function and return. Restore the original
                 # warning behavior during original function execution, since autologging is being
                 # skipped
-                with set_non_mlflow_warnings_behavior_for_current_thread(
+                with NonMlflowWarningsBehaviorForCurrentThread(
                     disable_warnings=False,
                     reroute_warnings=False,
                 ):
@@ -483,55 +413,26 @@ def safe_patch(
             # The active MLflow run (if any) associated with patch code execution
             patch_function_run_for_testing = None
             # The exception raised during executing patching function
-            patch_function_exception = None
-
-            def try_log_autologging_event(log_fn, *args):
-                try:
-                    log_fn(*args)
-                except Exception as e:
-                    _logger.debug(
-                        "Failed to log autologging event via '%s'. Exception: %s",
-                        log_fn,
-                        e,
-                    )
-
-            def call_original_fn_with_event_logging(original_fn, og_args, og_kwargs):
-                try:
-                    try_log_autologging_event(
-                        AutologgingEventLogger.get_logger().log_original_function_start,
-                        session,
-                        destination,
-                        function_name,
-                        og_args,
-                        og_kwargs,
-                    )
-                    original_fn_result = original_fn(*og_args, **og_kwargs)
-
-                    try_log_autologging_event(
-                        AutologgingEventLogger.get_logger().log_original_function_success,
-                        session,
-                        destination,
-                        function_name,
-                        og_args,
-                        og_kwargs,
-                    )
-                    return original_fn_result
-                except Exception as original_fn_e:
-                    try_log_autologging_event(
-                        AutologgingEventLogger.get_logger().log_original_function_error,
-                        session,
-                        destination,
-                        function_name,
-                        og_args,
-                        og_kwargs,
-                        original_fn_e,
-                    )
-
-                    nonlocal failed_during_original
-                    failed_during_original = True
-                    raise
+            patch_error = None
 
             with _AutologgingSessionManager.start_session(autologging_integration) as session:
+                event_logger = AutologgingEventLoggerWrapper(session, destination, function_name)
+
+                def call_original_fn_with_event_logging(original_fn, og_args, og_kwargs):
+                    try:
+                        event_logger.log_original_function_start(og_args, og_kwargs)
+
+                        original_fn_result = original_fn(*og_args, **og_kwargs)
+
+                        event_logger.log_original_function_success(og_args, og_kwargs)
+                        return original_fn_result
+                    except Exception as e:
+                        event_logger.log_original_function_error(og_args, og_kwargs, e)
+
+                        nonlocal failed_during_original
+                        failed_during_original = True
+                        raise
+
                 try:
 
                     def call_original(*og_args, **og_kwargs):
@@ -553,7 +454,7 @@ def safe_patch(
                                 # determine whether or not the patch implementation created
                                 # a run and perform validation if necessary
                                 nonlocal patch_function_run_for_testing
-                                patch_function_run_for_testing = mlflow.active_run()
+                                patch_function_run_for_testing = active_run()
 
                             nonlocal original_has_been_called
                             original_has_been_called = True
@@ -563,7 +464,7 @@ def safe_patch(
                             # during original function execution, even if silent mode is enabled
                             # (`silent=True`), since these warnings originate from the ML framework
                             # or one of its dependencies and are likely relevant to the caller
-                            with set_non_mlflow_warnings_behavior_for_current_thread(
+                            with NonMlflowWarningsBehaviorForCurrentThread(
                                 disable_warnings=False,
                                 reroute_warnings=False,
                             ):
@@ -577,33 +478,16 @@ def safe_patch(
                     # the signature of the `original` argument during execution
                     call_original = update_wrapper_extended(call_original, original)
 
-                    try_log_autologging_event(
-                        AutologgingEventLogger.get_logger().log_patch_function_start,
-                        session,
-                        destination,
-                        function_name,
-                        args,
-                        kwargs,
-                    )
+                    event_logger.log_patch_function_start(args, kwargs)
 
-                    if patch_is_class:
-                        patch_function.call(call_original, *args, **kwargs)
-                    else:
-                        patch_function(call_original, *args, **kwargs)
+                    patch_function(call_original, *args, **kwargs)
 
                     session.state = "succeeded"
+                    event_logger.log_patch_function_success(args, kwargs)
 
-                    try_log_autologging_event(
-                        AutologgingEventLogger.get_logger().log_patch_function_success,
-                        session,
-                        destination,
-                        function_name,
-                        args,
-                        kwargs,
-                    )
                 except Exception as e:
                     session.state = "failed"
-                    patch_function_exception = e
+                    patch_error = e
                     # Exceptions thrown during execution of the original function should be
                     # propagated to the caller. Additionally, exceptions encountered during test
                     # mode should be reraised to detect bugs in autologging implementations
@@ -613,9 +497,9 @@ def safe_patch(
                 if is_testing() and not preexisting_run_for_testing:
                     # If an MLflow run was created during the execution of patch code, verify that
                     # it is no longer active and that it contains expected autologging tags
-                    assert (
-                        not mlflow.active_run()
-                    ), f"Autologging integration {autologging_integration} leaked an active run"
+                    assert not active_run(), (
+                        f"Autologging integration {autologging_integration} leaked an active run"
+                    )
                     if patch_function_run_for_testing:
                         _validate_autologging_run(
                             autologging_integration, patch_function_run_for_testing.info.run_id
@@ -632,22 +516,153 @@ def safe_patch(
                     # If original function failed, we don't call `log_patch_function_error`
                     # even if `patch_function_exception` exists, because original function failure
                     # means there's some error in user code (e.g. user provide wrong arguments)
-                    if patch_function_exception is not None and not failed_during_original:
-                        try_log_autologging_event(
-                            AutologgingEventLogger.get_logger().log_patch_function_error,
-                            session,
-                            destination,
-                            function_name,
-                            args,
-                            kwargs,
-                            patch_function_exception,
+                    if patch_error is not None and not failed_during_original:
+                        event_logger.log_patch_function_error(args, kwargs, patch_error)
+                        _logger.warning(_ERROR_MSG.format(autologging_integration, patch_error))
+
+    async def async_safe_patch_function(*args, **kwargs):
+        """
+        Async version of safe_patch_function.
+
+        This code brainlessly copies the synchronous version of the function, but with async
+        context managers and async functions. This is done to avoid the risk of introducing
+        any bugs or regressions in the async version of the function. Note that we need to
+        be really careful here, because autologging is enabled by-default in DBR/MLR, hence
+        any bug here can break users' workload without them taking any action.
+
+        That said, some long comments are omitted in this version to avoid redundancy. If
+        you want to understand the context of the code better, please refer to the
+        synchronous version as well.
+        """
+        is_silent_mode = get_autologging_config(autologging_integration, "silent", False)
+        async with (
+            MlflowEventsAndWarningsBehaviorGlobally(
+                reroute_warnings=True,
+                disable_event_logs=is_silent_mode,
+                disable_warnings=is_silent_mode,
+            ),
+            NonMlflowWarningsBehaviorForCurrentThread(
+                disable_warnings=is_silent_mode,
+                reroute_warnings=True,
+            ),
+        ):
+            if is_testing():
+                preexisting_run_for_testing = active_run()
+
+            # Whether or not to exclude autologged content from user-created fluent runs
+            # (i.e. runs created manually via `mlflow.start_run()`)
+            exclusive = get_autologging_config(autologging_integration, "exclusive", False)
+            user_created_fluent_run_is_active = (
+                active_run() and not _AutologgingSessionManager.active_session()
+            )
+            active_session_failed = (
+                _AutologgingSessionManager.active_session() is not None
+                and _AutologgingSessionManager.active_session().state == "failed"
+            )
+
+            if (
+                active_session_failed
+                or autologging_is_disabled(autologging_integration)
+                or (user_created_fluent_run_is_active and exclusive)
+                or (
+                    mlflow.utils.autologging_utils._AUTOLOGGING_GLOBALLY_DISABLED
+                    and autologging_integration
+                )
+            ):
+                async with NonMlflowWarningsBehaviorForCurrentThread(False, False):
+                    return await original(*args, **kwargs)
+
+            original_has_been_called = False
+            original_result = None
+            failed_during_original = False
+            patch_function_run_for_testing = None
+            patch_error = None
+
+            async with _AutologgingSessionManager.astart_session(
+                autologging_integration
+            ) as session:
+                event_logger = AutologgingEventLoggerWrapper(session, destination, function_name)
+
+                async def call_original_fn_with_event_logging(original_fn, og_args, og_kwargs):
+                    try:
+                        event_logger.log_original_function_start(og_args, og_kwargs)
+                        original_fn_result = await original_fn(*og_args, **og_kwargs)
+                        event_logger.log_original_function_success(og_args, og_kwargs)
+                        return original_fn_result
+                    except Exception as e:
+                        event_logger.log_original_function_error(og_args, og_kwargs, e)
+                        nonlocal failed_during_original
+                        failed_during_original = True
+                        raise
+
+                try:
+
+                    async def call_original(*og_args, **og_kwargs):
+                        async def _original_fn(*_og_args, **_og_kwargs):
+                            if is_testing():
+                                _validate_args(
+                                    autologging_integration,
+                                    function_name,
+                                    args,
+                                    kwargs,
+                                    og_args,
+                                    og_kwargs,
+                                )
+                                nonlocal patch_function_run_for_testing
+                                patch_function_run_for_testing = active_run()
+
+                            nonlocal original_has_been_called
+                            original_has_been_called = True
+
+                            nonlocal original_result
+                            async with NonMlflowWarningsBehaviorForCurrentThread(False, False):
+                                original_result = await original(*_og_args, **_og_kwargs)
+                                return original_result
+
+                        return await call_original_fn_with_event_logging(
+                            _original_fn, og_args, og_kwargs
                         )
 
-                        _logger.warning(
-                            "Encountered unexpected error during %s autologging: %s",
-                            autologging_integration,
-                            patch_function_exception,
+                    # Apply the name, docstring, and signature of `original` to `call_original`.
+                    # This is important because several autologging patch implementations inspect
+                    # the signature of the `original` argument during execution
+                    call_original = update_wrapper_extended(call_original, original)
+
+                    event_logger.log_patch_function_start(args, kwargs)
+
+                    await patch_function(call_original, *args, **kwargs)
+
+                    session.state = "succeeded"
+                    event_logger.log_patch_function_success(args, kwargs)
+
+                except Exception as e:
+                    session.state = "failed"
+                    patch_error = e
+                    # Exceptions thrown during execution of the original function should be
+                    # propagated to the caller. Additionally, exceptions encountered during test
+                    # mode should be reraised to detect bugs in autologging implementations
+                    if failed_during_original or is_testing():
+                        raise
+
+                if is_testing() and not preexisting_run_for_testing:
+                    # If an MLflow run was created during the execution of patch code, verify that
+                    # it is no longer active and that it contains expected autologging tags
+                    assert not active_run(), (
+                        f"Autologging integration {autologging_integration} leaked an active run"
+                    )
+                    if patch_function_run_for_testing:
+                        _validate_autologging_run(
+                            autologging_integration, patch_function_run_for_testing.info.run_id
                         )
+                try:
+                    if original_has_been_called:
+                        return original_result
+                    else:
+                        return await call_original_fn_with_event_logging(original, args, kwargs)
+                finally:
+                    if patch_error is not None and not failed_during_original:
+                        event_logger.log_patch_function_error(args, kwargs, patch_error)
+                        _logger.warning(_ERROR_MSG.format(autologging_integration, patch_error))
 
     if is_property_method:
         # Create a patched function (also property decorated)
@@ -681,6 +696,8 @@ def safe_patch(
         # Make unbound method `class.target_method` keep the same doc and signature
         get_bound_safe_patch_fn = update_wrapper_extended(get_bound_safe_patch_fn, original_fn.fget)
         safe_patch_obj = property(get_bound_safe_patch_fn)
+    elif is_async_function:
+        safe_patch_obj = update_wrapper_extended(async_safe_patch_function, original)
     else:
         safe_patch_obj = update_wrapper_extended(safe_patch_function, original)
 
@@ -703,15 +720,6 @@ def revert_patches(autologging_integration):
     _AUTOLOGGING_PATCHES.pop(autologging_integration, None)
 
 
-def is_langchain_callbacks_manager(callbacks):
-    try:
-        from langchain_core.callbacks.base import BaseCallbackManager
-    except ImportError:
-        return False
-
-    return isinstance(callbacks, BaseCallbackManager)
-
-
 # Represents an active autologging session using two fields:
 # - integration: the name of the autologging integration corresponding to the session
 # - id: a unique session identifier (e.g., a UUID)
@@ -742,6 +750,19 @@ class _AutologgingSessionManager:
             if prev_session is None:
                 cls._end_session()
 
+    @classmethod
+    @asynccontextmanager
+    async def astart_session(cls, integration):
+        try:
+            prev_session = cls._session
+            if prev_session is None:
+                session_id = uuid.uuid4().hex
+                cls._session = AutologgingSession(integration, session_id)
+            yield cls._session
+        finally:
+            if prev_session is None:
+                cls._end_session()
+
     @classmethod
     def active_session(cls):
         return cls._session
@@ -814,6 +835,8 @@ def _validate_autologging_run(autologging_integration, run_id):
         - The run has an autologging tag whose value is the name of the autologging integration
         - The run has a terminal status (e.g., KILLED, FAILED, FINISHED)
     """
+    from mlflow.tracking.client import MlflowClient
+
     client = MlflowClient()
     run = client.get_run(run_id)
     autologging_tag_value = run.data.tags.get(MLFLOW_AUTOLOGGING)
@@ -821,12 +844,12 @@ def _validate_autologging_run(autologging_integration, run_id):
         f"Autologging run with id {run_id} failed to set autologging tag with expected value. "
         f"Expected: '{autologging_integration}', Actual: '{autologging_tag_value}'"
     )
-    assert RunStatus.is_terminated(
-        RunStatus.from_string(run.info.status)
-    ), f"Autologging run with id {run_id} has a non-terminal status '{run.info.status}'"
+    assert RunStatus.is_terminated(RunStatus.from_string(run.info.status)), (
+        f"Autologging run with id {run_id} has a non-terminal status '{run.info.status}'"
+    )
 
 
-class ValidationExemptArgument(typing.NamedTuple):
+class ValidationExemptArgument(NamedTuple):
     """
     A NamedTuple representing the properties of an argument that is exempt from validation
 
@@ -840,9 +863,9 @@ class ValidationExemptArgument(typing.NamedTuple):
 
     autologging_integration: str
     function_name: str
-    type_function: typing.Callable
-    positional_argument_index: Optional[int] = None
-    keyword_argument_name: Optional[str] = None
+    type_function: Callable[..., Any]
+    positional_argument_index: int | None = None
+    keyword_argument_name: str | None = None
 
     def matches(
         self,
@@ -987,9 +1010,9 @@ def _validate_args(
         autologging_call_input, user_call_input
     ):
         length_diff = len(autologging_call_input) - len(user_call_input)
-        assert (
-            length_diff >= 0
-        ), f"{length_diff} expected inputs are missing from the call to the original function."
+        assert length_diff >= 0, (
+            f"{length_diff} expected inputs are missing from the call to the original function."
+        )
 
     def _assert_autologging_input_kwargs_are_superset(autologging_call_input, user_call_input):
         assert set(user_call_input.keys()).issubset(set(autologging_call_input.keys())), (
@@ -1022,10 +1045,10 @@ def _validate_args(
             _validate_new_input(autologging_call_input)
             return
 
-        assert type(autologging_call_input) == type(
-            user_call_input
-        ), "Type of input to original function '{}' does not match expected type '{}'".format(
-            type(autologging_call_input), type(user_call_input)
+        assert type(autologging_call_input) == type(user_call_input), (
+            "Type of input to original function '{}' does not match expected type '{}'".format(
+                type(autologging_call_input), type(user_call_input)
+            )
         )
 
         if type(autologging_call_input) in [list, tuple]:
@@ -1041,15 +1064,7 @@ def _validate_args(
             _assert_autologging_input_kwargs_are_superset(autologging_call_input, user_call_input)
             for key in autologging_call_input.keys():
                 _validate(autologging_call_input[key], user_call_input.get(key, None))
-        # NB: For LangChain autologging, we replace the callback manager that is passed by
-        # the user with the one we copied and injected our callbacks into. We cannot do this
-        # in-place to avoid side-effects, so create a new callback manager instance. It will
-        # fail at the strict equality check below, so we instead check that their handlers
-        # are effectively the compatible.
-        elif is_langchain_callbacks_manager(
-            autologging_call_input
-        ) and is_langchain_callbacks_manager(user_call_input):
-            _validate(autologging_call_input.handlers, user_call_input.handlers)
+
         else:
             assert (
                 autologging_call_input is user_call_input
