@@ -6,7 +6,7 @@ import sys
 import tempfile
 import uuid
 from pathlib import Path
-from typing import Literal, Optional
+from typing import Literal
 
 from packaging.version import Version
 
@@ -16,6 +16,7 @@ from mlflow.exceptions import MlflowException
 from mlflow.models.model import MLMODEL_FILE_NAME, Model
 from mlflow.utils import env_manager as em
 from mlflow.utils.conda import _PIP_CACHE_DIR
+from mlflow.utils.databricks_utils import is_in_databricks_runtime
 from mlflow.utils.environment import (
     _CONDA_ENV_FILE_NAME,
     _PYTHON_ENV_FILE_NAME,
@@ -243,11 +244,11 @@ def _create_virtualenv(
     local_model_path: Path,
     python_env: _PythonEnv,
     env_dir: Path,
-    pyenv_root_dir: Optional[str] = None,
+    pyenv_root_dir: str | None = None,
     env_manager: Literal["virtualenv", "uv"] = em.UV,
-    extra_env: Optional[dict[str, str]] = None,
+    extra_env: dict[str, str] | None = None,
     capture_output: bool = False,
-    pip_requirements_override: Optional[list[str]] = None,
+    pip_requirements_override: list[str] | None = None,
 ):
     if env_manager not in {em.VIRTUALENV, em.UV}:
         raise MlflowException.invalid_parameter_value(
@@ -280,7 +281,7 @@ def _create_virtualenv(
             f"version {python_env.python} using uv"
         )
         env_creation_cmd = ["uv", "venv", env_dir, f"--python={python_env.python}"]
-        install_deps_cmd_prefix = "uv pip install --prerelease=allow"
+        install_deps_cmd_prefix = "uv pip install"
         if _MLFLOW_TESTING.get():
             os.environ["RUST_LOG"] = "uv=debug"
     with remove_on_error(
@@ -371,13 +372,14 @@ _VIRTUALENV_ENVS_DIR = "virtualenv_envs"
 _PYENV_ROOT_DIR = "pyenv_root"
 
 
-def _get_or_create_virtualenv(  # noqa: D417
+def _get_or_create_virtualenv(
     local_model_path,
     env_id=None,
     env_root_dir=None,
     capture_output=False,
-    pip_requirements_override: Optional[list[str]] = None,
+    pip_requirements_override: list[str] | None = None,
     env_manager: Literal["virtualenv", "uv"] = em.UV,
+    extra_envs: dict[str, str] | None = None,
 ):
     """Restores an MLflow model's environment in a virtual environment and returns a command
     to activate it.
@@ -393,6 +395,8 @@ def _get_or_create_virtualenv(  # noqa: D417
             the environment (upgrade if already installed).
         env_manager: Specifies the environment manager to use to create the environment.
             Defaults to "uv".
+        extra_envs: If specified, a dictionary of extra environment variables will be passed to the
+            environment creation command.
 
             .. tip::
                 It is highly recommended to use "uv" as it has significant performance improvements
@@ -420,15 +424,30 @@ def _get_or_create_virtualenv(  # noqa: D417
             pyenv_root_path.mkdir(parents=True, exist_ok=True)
             pyenv_root_dir = str(pyenv_root_path)
 
-    virtual_envs_root_path = (
-        Path(env_root_dir) / _VIRTUALENV_ENVS_DIR
-        if env_root_dir is not None
-        else Path(_get_mlflow_virtualenv_root())
-    )
     virtual_envs_root_path.mkdir(parents=True, exist_ok=True)
     env_name = _get_virtualenv_name(python_env, local_model_path, env_id)
     env_dir = virtual_envs_root_path / env_name
-    extra_env = _get_virtualenv_extra_env_vars(env_root_dir)
+    try:
+        env_dir.exists()
+    except PermissionError:
+        if is_in_databricks_runtime():
+            # Updating env_name only doesn't work because the cluster may not have
+            # permission to access the original virtual_envs_root_path
+            virtual_envs_root_path = (
+                Path(env_root_dir) / f"{_VIRTUALENV_ENVS_DIR}_{uuid.uuid4().hex[:8]}"
+            )
+            virtual_envs_root_path.mkdir(parents=True, exist_ok=True)
+            env_dir = virtual_envs_root_path / env_name
+        else:
+            _logger.warning(
+                f"Existing virtual environment directory {env_dir} cannot be accessed "
+                "due to permission error. Check the permissions of the directory and "
+                "try again. If the issue persists, consider cleaning up the directory manually."
+            )
+            raise
+
+    extra_envs = extra_envs or {}
+    extra_envs |= _get_virtualenv_extra_env_vars(env_root_dir)
 
     # Create an environment
     return _create_virtualenv(
@@ -437,7 +456,7 @@ def _get_or_create_virtualenv(  # noqa: D417
         env_dir=env_dir,
         pyenv_root_dir=pyenv_root_dir,
         env_manager=env_manager,
-        extra_env=extra_env,
+        extra_env=extra_envs,
         capture_output=capture_output,
         pip_requirements_override=pip_requirements_override,
     )
